/***************************************************************************
Name: CookieSafe
Description: Control cookie permissions.
Author: Ron Beckman
Homepage: http://forum.softwareblaze.com

Copyright (C) 2007  Ron Beckman

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

Free Software Foundation, Inc.
51 Franklin Street
Fifth Floor
Boston, MA  02110-1301
USA
***************************************************************************/



var cookiesafeCleanCookies = {

	_hosts   : [],
	_cookies : [],

	scanningMeter: null,

	init: function() {
		this.scanningMeter = setInterval(cookiesafeCleanCookies.displayMeter,250);

		this.downloadBlocklist();
	},

	displayMeter: function() {
		var meter = document.getElementById('scanning-meter');
		var dots = meter.getAttribute('value');
		meter.setAttribute('value',(dots.length >= 40) ? '. ' : dots+'. ');
	},

	removeSelected: function() {
		var start = {};
		var end = {};
		var items = [];
		var cookies = [];
		var mngr = this.getCookieManager();
		var tree = document.getElementById('cookies-tree');
		var child = document.getElementById('cookies-child');
		var range = tree.view.selection.getRangeCount();

		for (var i=0; i<range; ++i) {
			tree.view.selection.getRangeAt(i,start,end);
			for (var j=start.value; j<=end.value; ++j) {
				cookies.push(this.getRowData(j));
				items.push(tree.contentView.getItemAtIndex(j));
			}
		}

		for (var i=0; i<cookies.length; ++i) {
			try {
				mngr.remove(cookies[i][0],cookies[i][2],cookies[i][1],false);
			} catch(e) {
				continue;
			}
		}

		for (var i=0; i<items.length; ++i) {
			tree.view.selection.clearRange(i,i);
			child.removeChild(items[i]);
		}

		this.updateTotal();
	},

	removeAll: function() {
		var items = [];
		var cookies = [];
		var mngr = this.getCookieManager();
		var tree = document.getElementById('cookies-tree');
		var child = document.getElementById('cookies-child');

		for (var i=0; i<tree.view.rowCount; ++i) {
			cookies.push(this.getRowData(i));
			items.push(tree.contentView.getItemAtIndex(i));
		}

		for (var i=0; i<cookies.length; ++i) {
			try {
				mngr.remove(cookies[i][0],cookies[i][2],cookies[i][1],false);
			} catch(e) {
				continue;
			}
		}

		for (var i=0; i<items.length; ++i) {
			tree.view.selection.clearRange(i,i);
			child.removeChild(items[i]);
		}

		this.updateTotal();
	},

	sortCol: function(sortBy) {
		//save the name of the column we're sorting
		var tree = document.getElementById('cookies-tree');
		var lastClicked = tree.getAttribute('lastClicked');

		//determine whether to sort a to z
		var col = document.getElementById(sortBy);
		var atoz = col.getAttribute('atoz');
		if (sortBy==lastClicked) {
			atoz = (atoz && atoz=='true') ? false : true;
		} else {
			atoz = (!atoz || atoz=='true') ? true : false;
		}
		col.setAttribute('atoz',atoz);

		tree.setAttribute('lastClicked',sortBy);

		this.populateTree(sortBy,atoz);
	},

	downloadBlocklist: function() {
		var prefs = this.getPrefs();
		var url = prefs.getCharPref('updateUrl');

		var httpRequest = new XMLHttpRequest();
		httpRequest.overrideMimeType('text/xml');
		httpRequest.onreadystatechange = function() { cookiesafeCleanCookies.processBlocklist(httpRequest); };
		httpRequest.open('GET', url, true);
		httpRequest.send(null);
	},

	processBlocklist: function(req) {
		if (req.readyState == 4) {
			/*for some reason if the user is not connected to the network the status
			property will throw an error when it's accessed. Although when you query
			it using ('status' in req) it returns true? At this point the only thing
			to do is enclose the req.status command in try-catch brackets.*/

			try { var status = req.status; } catch(e) { return false; }

			// if the user is updating via a local file url the status will be 0
			if (status == 200 || status == 0) {
				var hosts = [];
				var doc = req.responseXML;
				var elmnts = doc.getElementsByTagName('host');

				//create _hosts array of blocklist hosts
				for (var i=0; i<elmnts.length; ++i) {
					this._hosts.push(elmnts[i].textContent);
				}

				//create _cookies array of spyware cookies
				this.scanCookies();

				//determine which column to sort by
				var tree = document.getElementById('cookies-tree');
				var sortBy = tree.getAttribute('lastClicked');
				if (!sortBy) {
					sortBy = 'host';
					tree.setAttribute('lastClicked',sortBy);
				}

				//determine whether to sort a to z
				var col = document.getElementById(sortBy);
				var atoz = col.getAttribute('atoz');
				atoz = (!atoz || atoz=='true') ? true : false;
				col.setAttribute('atoz',atoz);

				this.populateTree(sortBy,atoz);

				//remove the scanning meter
				clearInterval(this.scanningMeter);
				document.getElementById('scanningBox').setAttribute('hidden','true');
			}
		}
		return false;
	},

	scanCookies: function() {
		var ck,base;
		var cs = this.getCS();
		var mngr = this.getCookieManager();
		var cks = mngr.enumerator;
		while (('hasMoreElements' in cks && cks.hasMoreElements()) ||
			 ('hasMore' in cks && cks.hasMore())) {
			ck = cks.getNext();
			ck.QueryInterface(Components.interfaces.nsICookie);
			ck.QueryInterface(Components.interfaces.nsICookie2);
			base = cs.removeSub(ck.rawHost);
			if (this._hosts.indexOf(base) != -1) {
				this._cookies.push(ck);
			}
		}
	},

	populateTree: function(sortBy,atoz) {
		var expires,ck,sec;
		var cookies = [];
		for (var i=0; i<this._cookies.length; ++i) {
			ck = this._cookies[i];

			//set expiration label in xx/xx/xx format
			expires = this.formatExpiration(ck.expires);

			if (ck.isSecure) {
				sec = this.getStr('cookiesafe.lYes');
			} else {
				sec = this.getStr('cookiesafe.lNo');
			}

			cookies.push([ck[sortBy],ck.host,ck.path,ck.name,ck.value,sec,expires]);
		}

		cookies.sort();
		if (!atoz) cookies.reverse();

		// make sure no rows are currently selected
		var tree = document.getElementById('cookies-tree');
		tree.view.selection.clearSelection();

		// remove all tree items
		var child = document.getElementById('cookies-child');
		while (child.firstChild) {
			child.removeChild(child.firstChild);
		}

		for (var i=0; i<cookies.length; ++i) {
			child.appendChild(this.createItem(cookies[i]));
		}

		this.updateTotal();
	},

	updateTotal: function() {
		// display total number of cookies
		var tree = document.getElementById('cookies-tree');
		var total = document.getElementById('total');
		total.setAttribute('value',this.getStr('cookiesafe.lTotal')+' '+tree.view.rowCount);
	},

	createItem: function(line) {
		var item = document.createElement('treeitem');
		var row = document.createElement('treerow');

		//index 0 is the sortBy value
		for (var i=1; i<line.length; ++i) {
			var cell = document.createElement('treecell');
			cell.setAttribute('label',line[i]);
			row.appendChild(cell);
		}

		item.appendChild(row);
		return item;
	},

	formatExpiration: function(num) {
		var exp,dt;

		if (num == 0) {
			exp = this.getStr('cookiesafe.lSession');
		} else {
			dt = new Date();
			dt.setTime(num * 1000);
			exp = (dt.getMonth()+1)+'/'+dt.getDate()+'/'+dt.getFullYear();
		}

		return exp;
	},

	getStr: function(name) {
		return document.getElementById("cookiesafe-bundle").getString(name);
	},

	getPrefs: function() {
		return Components.classes["@mozilla.org/preferences-service;1"].
		getService(Components.interfaces.nsIPrefService).
		getBranch("cookiesafe.");
	},

	getCS: function() {
		return Components.classes['@mozilla.org/CookieSafe;1'].
		createInstance(Components.interfaces.nsICookieSafe);
	},

	getCookieManager: function() {
		return Components.classes["@mozilla.org/cookiemanager;1"].
		getService(Components.interfaces.nsICookieManager);
	},

	getRowData: function(index) {
		var row = [];
		var tree = document.getElementById('cookies-tree');

		try {
			for (var i=0; i<tree.columns.count; ++i) {
				row.push(tree.view.getCellText(index,tree.columns.getColumnAt(i)));
			}
		} catch(e) {
			return [];
		}

		return row;
	}
};